# Generated by roxygen2: do not edit by hand

export(ds.arrange)
export(ds.as_tibble)
export(ds.bind_cols)
export(ds.bind_rows)
export(ds.case_when)
export(ds.distinct)
export(ds.filter)
export(ds.group_by)
export(ds.group_keys)
export(ds.if_else)
export(ds.mutate)
export(ds.rename)
export(ds.select)
export(ds.slice)
export(ds.ungroup)
importFrom(DSI,datashield.aggregate)
importFrom(DSI,datashield.assign)
importFrom(DSI,datashield.connections_find)
importFrom(assertthat,assert_that)
importFrom(cli,cli_abort)
importFrom(rlang,enquo)
importFrom(rlang,quo_text)
importFrom(rlang,sym)
